unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    TestButton: TButton;
    ResultMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TestButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOAIO;

const
	ResetOffset = $00;
	IRQEnableOffset = $01;
	DACOffset = $04;
	BaseClockOffset = $0C;
	DivisorOffset = $10;
	ADCRangeOffset = $18;
	AFLevelOffset = $20;
	ADCDepthOffset = $28;
	ADCDataOffset: array[0..1] of TUInt16 = ($30, $34);
	ADCStartStopOffset = $38;
	//ADCImmediateOffset = $3A;
	VersionOffset = $68;
	//ADCDataOffset: array[0..1] of TUInt32 = ($4000, $8000);
	dSequencer = $4;

	ADCCmd_Base = $3884A;
	ADCCmd_ChShift = 12;
	ADCStatus_ChShift = 20;

var
  TicksPerSec, TicksPerMicrosec: Int64;

procedure uDelay(Microseconds: Integer);
var
  EndTime, Naui: Int64;

begin
  QueryPerformanceCounter(EndTime);
  EndTime := EndTime + TicksPerMicrosec * Microseconds;
  repeat
    QueryPerformanceCounter(Naui);
  until (Naui - EndTime) >= 0;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  //`@
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  //`@
end;

procedure TMainForm.TestButtonClick(Sender: TObject);
const
  iBoard = 0;

var
  TestCount, PassCount, ErrCount, ReadbackErrCount, Ch: Integer;
  ControlValue, ReadControlValue: TUInt32;
  ADCFIFODepth: TUInt32;
  ADCDataRaw: TUInt32;
  RunResult: String;
  DeviceID, ChCount: TUInt32;

begin
  TestCount := 0;
  PassCount := 0;
  ErrCount := 0;
  ReadbackErrCount := 0;

  BRD_GetDeviceID(iBoard, @DeviceID);
  case DeviceID of
    $C0E8, $C0E9, $C0EA, //mPCIe-AIO16-16x
    $80E8, $80E9, $80EA, //mPCIe-AI16-16x
    $C058, $C059, $C05A, //mPCIe-AIO12-16x
    $8058, $8059, $805A: //mPCIe-AI12-16x
    ChCount := 16;

    $C0EC, $C0ED, $C0EE, //mPCIe-ADIO16-16x
    $80EC, $80ED, $80EE, //mPCIe-ADI16-16x
    $C05C, $C05D, $C05E, //mPCIe-ADIO12-16x
    $805C, $805D, $805E: //mPCIe-ADI12-16x
    ChCount := 8;

    $C2E8, $C2E9, $C2EA, //mPCIe-AIO16-16x prototype
    $82E8, $82E9, $82EA, //mPCIe-AI16-16x prototype
    $C258, $C259, $C25A, //mPCIe-AIO12-16x prototype
    $8258, $8259, $825A: //mPCIe-AI12-16x prototype
    ChCount := 16;

    else Exit;
  end;

  ResultMemo.Lines.Clear;
  ResultMemo.Lines.Add('Control value used is ' + IntToHex(ADCCmd_Base, 5) + ' through ' + IntToHex(ADCCmd_Base or ($F shl ADCCmd_ChShift), 5));

  while TestCount < 1000 do begin
    BRD_Reset(iBoard);

    //Sleep(1);

    RunResult := '';
    PIO_Out32(iBoard, DivisorOffset, 0); //setting ADC Rate Divisor to zero selects software start ADC mode
    for Ch := 0 to 7 do begin
            ControlValue := ADCCmd_Base or (Ch shl ADCCmd_ChShift);
            PIO_Out32(iBoard, ADCStartStopOffset, ControlValue); //start one conversion on selected channel
            uDelay(10); //must not write to +38 faster than once every 10 microseconds in Software Start mode
            ReadControlValue := PIO_In32(iBoard, ADCStartStopOffset);
            if (ControlValue and $0000FFFF) <> (ReadControlValue and $0000FFFF) then begin
                    Inc(ReadbackErrCount);
                    RunResult := RunResult + IntToHex(ReadControlValue, 8) + '/ ' + IntToHex(ControlValue, 8);
            end;
    end;

    if ChCount = 16 then
            for Ch := 0 to 7 do begin
                    ControlValue := ADCCmd_Base or (Ch shl ADCCmd_ChShift);
                    PIO_Out32(iBoard, ADCStartStopOffset + dSequencer, ControlValue); //start one conversion on selected channel of second ADC
                    uDelay(10); //must not write to +3C faster than once every 10 microseconds in Software Start mode
                    ReadControlValue := PIO_In32(iBoard, ADCStartStopOffset + dSequencer);
                    if (ControlValue and $0000FFFF) <> (ReadControlValue and $0000FFFF) then begin
                            Inc(ReadbackErrCount);
                            RunResult := RunResult + IntToHex(ReadControlValue, 8) + '/' + IntToHex(ControlValue, 8) + ' ';
                    end;
            end
    ;

    ADCFIFODepth := PIO_In32(iBoard, ADCDepthOffset);
    if ADCFIFODepth <> ChCount then
            Inc(ErrCount)
    else 
            Inc(PassCount)
    ;
    Inc(TestCount);
    for Ch := 1 to ADCFIFODepth do begin //read and display data from FIFO
            ADCDataRaw := PIO_In32(iBoard, ADCDataOffset[0]);
            RunResult := RunResult + IntToHex((ADCDataRaw shr ADCStatus_ChShift) and $F, 1) + ' ';
    end;
    ResultMemo.Lines.Add(RunResult);
    Application.ProcessMessages;
  end;

  ResultMemo.Lines.Add('');
  ResultMemo.Lines.Add(IntToStr(ErrCount) + 'h failures, ' + IntToStr(PassCount) + ' passes. ' + FloatToStr(ErrCount / TestCount * 100) + '% failure.  Control readback failures: ' + IntToStr(ReadbackErrCount));
end;

begin
  QueryPerformanceFrequency(TicksPerSec);
  TicksPerMicrosec := TicksPerSec div 1000000;
end.
